'----------------------------------------------------------------------------------------------------------------------
'
' Demo and test harness for heater_fft
'
' Michael Rychlik 2010-12-21
'
'----------------------------------------------------------------------------------------------------------------------
OBJ
    'Set your platform configuration in userdef.spin
    'Ideally, your userdefs.spin can live in the library path,
    'and you can simply remove a new package's userdefs.spin
    'to enable your own definitions.
    def  : "userdefs.spin"

    fft  : "heater_fft.spin"

    'User interface port
    ser  : "FullDuplexSerialPlus"

'----------------------------------------------------------------------------------------------------------------------

'----------------------------------------------------------------------------------------------------------------------
CON
    _clkmode      = def#_clkmode
    _xinfreq      = def#_xinfreq

#define PASM_BUTTERFLIES
'#define SPIN_BUTTERFLIES

'----------------------------------------------------------------------------------------------------------------------

'----------------------------------------------------------------------------------------------------------------------
VAR
    'Pointers to bx and by FFT buffers
    long bx
    long by
'----------------------------------------------------------------------------------------------------------------------

'----------------------------------------------------------------------------------------------------------------------
PUB testFFT | startTime, endTime
'Test harness for the FFT
'
    'Start the terminal
    ser.start(def#conRxPin, def#conTxPin, def#conMode, def#conBaud)
    ser.str(string("heater_fft v0.5"))
    newline

    fft.start

    'Get pointers to internal FFT buffers
    bx := fft.getbx
    by := fft.getby

    'Input some data
    fillInput

    'Radix-2 Decimation In Time, the bit-reversal step.
    'This take only ~16% of the total FFT time.
    fft.decimate

    'The actual Fourier transform work.
    startTime := cnt
    fft.butterflies

    endTime := cnt

    'Print resulting spectrum
    printSpectrum

    ser.str(string("1024 point butterfly run time = "))
    ser.dec((endTime - startTime) / (clkfreq / 1000))
    ser.str(string("ms"))
    newline
    repeat

PUB printReals | i
    repeat i from 0 to fft#FFT_SIZE - 1
        ser.dec(long[@bx + i * 4])
        newline

PUB printSpectrum  | i, real, imag, magnitude
'Spectrum is available in first half of the buffers after FFT.
    repeat i from 0 to (fft#FFT_SIZE / 2)
        'Frequency magnitde is square root of cos part sqaured plus sin part squared
        real := long[bx + i * 4]
        imag := long[by + i * 4]
        magnitude := ^^((real * real) / 4096) + ((imag * imag) / 4096)

        'Scale back down to range 0 to 2047
        magnitude /= 32
        ser.dec(magnitude)
        newline

PUB fillInput | k, r, sx, sy
'Fill buffer bx with samples of of an imput signal and clear by.
'WARNING: Keep the peek amplitude < 2048.
    repeat k from 0 to (fft#FFT_SIZE - 1)
        'Two frequencies of the waveform defined in input
        sx:= bx + k * 4
        sy:= by + k * 4
        long[sx] := (~~input[(3*k) // 16] / 32) - 1
        long[sx] += (~~input[(5*k) // 16] / 32) - 1

        'The highest frequency
        if (k & 1)
          long[sx] += (2048 / 32) - 1
        else
          long[sx] += -2048 / 32
        'A DC level
        long[sx] += (2048 / 32) - 1

        'Clear Y array.
        long[sy] := 0

PUB newline
    ser.tx(13)
    ser.tx(10)
'----------------------------------------------------------------------------------------------------------------------

'----------------------------------------------------------------------------------------------------------------------
DAT
'For testing define 16 samples  of an input wave form here.
input word 4096, 3784, 2896, 1567, 0, -1567, -2896, -3784, -4096, -3784, -2896, -1567, 0, 1567, 2896, 3784
'----------------------------------------------------------------------------------------------------------------------

'----------------------------------------------------------------------------------------------------------------------
'The end.

